@@ -6,40 +6,63 @@ qualifying view to call in response to an RPC method.
 """
 import inspect
 
-from pyramid.interfaces import IRequest
-from pyramid.interfaces import IRouteRequest
-from pyramid.interfaces import IView
-from pyramid.interfaces import IViewClassifier
 from pyramid.interfaces import IViewMapperFactory
 
-from zope.interface import implements
-from zope.interface import providedBy
+from zope.interface import implementer
 
+from pyramid_rpc.compat import PY3
 
-def view_lookup(request, method):
-    """Lookup and return a view based on the request, context, and
-    method name
+if PY3: # pragma: no cover
+    def _inspect_ob(f):
+        im = False
 
-    This function will use the current routes name to locate the
-    view using Pyramid's view lookup machinery.
+        if hasattr(f, '__func__'):
+            im = True
 
-    """
-    registry = request.registry
-    adapters = registry.adapters
+        elif not hasattr(f, '__defaults__'):
+            if hasattr(f, '__call__'):
+                f = f.__call__
+                if hasattr(f, '__func__'):
+                    im = True
 
-    # Hairy view lookup stuff below, woo!
-    request_iface = registry.queryUtility(
-        IRouteRequest, name=request.matched_route.name,
-        default=IRequest)
-    context_iface = providedBy(request.context)
-    view_callable = adapters.lookup(
-        (IViewClassifier, request_iface, context_iface),
-        IView, name=method, default=None)
-    return view_callable
+        if im:
+            f = f.__func__
+            c = f.__code__
+            defaults = f.__defaults__
+            names = c.co_varnames[1:c.co_argcount]
+        else:
+            c = f.__code__
+            defaults = f.__defaults__
+            names = c.co_varnames[:c.co_argcount]
 
+        return names, defaults
+else:
+    def _inspect_ob(f):
+        im = False
 
+        if hasattr(f, 'im_func'):
+            im = True
+
+        elif not hasattr(f, 'func_defaults'):
+            if hasattr(f, '__call__'):
+                f = f.__call__
+                if hasattr(f, 'im_func'):
+                    im = True
+
+        if im:
+            f = f.im_func
+            c = f.func_code
+            defaults = f.func_defaults
+            names = c.co_varnames[1:c.co_argcount]
+        else:
+            c = f.func_code
+            defaults = f.func_defaults
+            names = c.co_varnames[:c.co_argcount]
+
+        return names, defaults
+
+@implementer(IViewMapperFactory)
 class MapplyViewMapper(object):
-    implements(IViewMapperFactory)
 
     def __init__(self, **kw):
         self.attr = kw.get('attr')
@@ -89,28 +112,7 @@ class MapplyViewMapper(object):
         return mapped_view
 
     def mapply(self, ob, positional, keyword):
-
-        f = ob
-        im = False
-
-        if hasattr(f, 'im_func'):
-            im = True
-
-        elif not hasattr(f, 'func_defaults'):
-            if hasattr(f, '__call__'):
-                f = f.__call__
-                if hasattr(f, 'im_func'):
-                    im = True
-
-        if im:
-            f = f.im_func
-            c = f.func_code
-            defaults = f.func_defaults
-            names = c.co_varnames[1:c.co_argcount]
-        else:
-            defaults = f.func_defaults
-            c = f.func_code
-            names = c.co_varnames[:c.co_argcount]
+        names, defaults = _inspect_ob(ob)
 
         nargs = len(names)
         args = []
