@@ -1,11 +1,16 @@
+import json
 import logging
 
 import venusian
-from pyramid.compat import json
 from pyramid.exceptions import ConfigurationError
-from pyramid.httpexceptions import HTTPForbidden
-from pyramid.httpexceptions import HTTPNotFound
+from pyramid.httpexceptions import (
+    HTTPForbidden,
+    HTTPNoContent,
+    HTTPNotFound,
+)
+from pyramid.renderers import null_renderer
 from pyramid.response import Response
+from pyramid.security import NO_PERMISSION_REQUIRED
 
 from pyramid_rpc.api import MapplyViewMapper
 from pyramid_rpc.api import ViewMapperArgsInvalid
@@ -15,18 +20,25 @@ log = logging.getLogger(__name__)
 
 
 class JsonRpcError(Exception):
-    code = None
-    message = None
+    code = -32603 # sane default
+    message = 'internal error' # sane default
+    data = None
 
-    def __init__(self, data=None):
-        self.data = data
+    def __init__(self, code=None, message=None, data=None):
+        if code is not None:
+            self.code = code
+        if message is not None:
+            self.message = message
+        if data is not None:
+            self.data = data
 
     def as_dict(self):
         """Return a dictionary representation of this object for
         serialization in a JSON-RPC response."""
         error = dict(code=self.code,
                      message=self.message)
-
+        if self.data is not None:
+            error['data'] = self.data
         return error
 
 
@@ -65,11 +77,10 @@ def jsonrpc_error_response(error, id=None):
         'jsonrpc': '2.0',
         'id': id,
         'error': error.as_dict(),
-    })
+    }).encode('utf-8')
 
     response = Response(body)
     response.content_type = 'application/json'
-    response.content_length = len(body)
     return response
 
 
@@ -77,6 +88,8 @@ def exception_view(exc, request):
     rpc_id = getattr(request, 'rpc_id', None)
     if isinstance(exc, JsonRpcError):
         fault = exc
+        log.debug('json-rpc error rpc_id:%s "%s"',
+                  rpc_id, exc.message)
     elif isinstance(exc, HTTPNotFound):
         fault = JsonRpcMethodNotFound()
         log.debug('json-rpc method not found rpc_id:%s "%s"',
@@ -95,29 +108,24 @@ def exception_view(exc, request):
     return jsonrpc_error_response(fault, rpc_id)
 
 
-def jsonrpc_renderer(info):
-    def _render(value, system):
-        request = system.get('request')
-        if request is not None:
-            rpc_id = getattr(request, 'rpc_id', None)
-            response = request.response
+def make_response(request, result):
+    rpc_id = getattr(request, 'rpc_id', None)
+    response = request.response
 
-            if rpc_id is None:
-                response.status = 204
-                del response.content_type
-                return ''
+    if rpc_id is None:
+        return HTTPNoContent()
 
-            ct = response.content_type
-            if ct == response.default_content_type:
-                response.content_type = 'application/json'
+    ct = response.content_type
+    if ct == response.default_content_type:
+        response.content_type = 'application/json'
 
-            out = {
-                'jsonrpc': '2.0',
-                'id': rpc_id,
-                'result': value,
-            }
-            return json.dumps(out)
-    return _render
+    out = {
+        'jsonrpc': '2.0',
+        'id': rpc_id,
+        'result': result,
+    }
+    response.body = json.dumps(out).encode('utf-8')
+    return response
 
 
 def setup_jsonrpc(request):
@@ -132,11 +140,18 @@ def setup_jsonrpc(request):
     request.rpc_version = body.get('jsonrpc')
 
     if request.rpc_version != '2.0':
+        log.debug('id:%s invalid rpc version %s',
+                  request.rpc_id, request.rpc_version)
         raise JsonRpcRequestInvalid
 
     if request.rpc_method is None:
+        log.debug('id:%s invalid rpc method %s',
+                  request.rpc_id, request.rpc_method)
         raise JsonRpcRequestInvalid
 
+    log.debug('handling id:%s method:%s',
+              request.rpc_id, request.rpc_method)
+
 
 def add_jsonrpc_endpoint(self, name, *args, **kw):
     """Add an endpoint for handling JSON-RPC.
@@ -160,7 +175,8 @@ def add_jsonrpc_endpoint(self, name, *args, **kw):
     predicates = kw.setdefault('custom_predicates', [])
     predicates.append(jsonrpc_endpoint_predicate)
     self.add_route(name, *args, **kw)
-    self.add_view(exception_view, route_name=name, context=Exception)
+    self.add_view(exception_view, route_name=name, context=Exception,
+                  permission=NO_PERMISSION_REQUIRED)
 
 
 def add_jsonrpc_method(self, view, **kw):
@@ -199,8 +215,15 @@ def add_jsonrpc_method(self, view, **kw):
         return getattr(request, 'rpc_method', None) == method
     predicates = kw.setdefault('custom_predicates', [])
     predicates.append(jsonrpc_method_predicate)
+    decorator = kw.get('decorator', lambda v: v)
+    def renderer(view):
+        def wrapper(context, request):
+            result = decorator(view)(context, request)
+            return make_response(request, result)
+        return wrapper
+    kw['decorator'] = renderer
+    kw['renderer'] = null_renderer
     kw.setdefault('mapper', MapplyViewMapper)
-    kw.setdefault('renderer', 'pyramid_rpc:jsonrpc')
     self.add_view(view, route_name=endpoint, **kw)
 
 
@@ -255,5 +278,5 @@ def includeme(config):
     """
     config.add_directive('add_jsonrpc_endpoint', add_jsonrpc_endpoint)
     config.add_directive('add_jsonrpc_method', add_jsonrpc_method)
-    config.add_renderer('pyramid_rpc:jsonrpc', jsonrpc_renderer)
-    config.add_view(exception_view, context=JsonRpcError)
+    config.add_view(exception_view, context=JsonRpcError,
+                    permission=NO_PERMISSION_REQUIRED)
