@@ -6,8 +6,8 @@ try:
     import pyamf
     from pyamf import remoting, util
 except ImportError: # pragma: nocover
-    from nose.plugins.skip import SkipTest
-    raise SkipTest("PyAMF not installed, skipping AMF tests.")
+    from nose import SkipTest
+    raise SkipTest
 
 class DummyLogging(object):
     def exception(self, *args):
