@@ -1,7 +1,7 @@
+import json
 import unittest
 
 from pyramid import testing
-from pyramid.compat import json
 
 from webtest import TestApp
 
@@ -14,7 +14,8 @@ class TestJSONRPCIntegration(unittest.TestCase):
         testing.tearDown()
 
     def _callFUT(self, app, method, params, id=5, version='2.0',
-                 path='/api/jsonrpc', content_type='application/json'):
+                 path='/api/jsonrpc', content_type='application/json',
+                 expect_error=False):
         body = {}
         if id is not None:
             body['id'] = id
@@ -26,10 +27,10 @@ class TestJSONRPCIntegration(unittest.TestCase):
             body['params'] = params
         resp = app.post(path, content_type=content_type,
                         params=json.dumps(body))
-        if id is not None:
+        if id is not None or expect_error:
             self.assertEqual(resp.status_int, 200)
             self.assertEqual(resp.content_type, 'application/json')
-            result = json.loads(resp.body)
+            result = resp.json
             self.assertEqual(result['jsonrpc'], '2.0')
             self.assertEqual(result['id'], id)
         else:
@@ -120,7 +121,7 @@ class TestJSONRPCIntegration(unittest.TestCase):
         app = config.make_wsgi_app()
         app = TestApp(app)
         result = self._callFUT(app, 'dummy', [2, 3], id=None)
-        self.assertEqual(result, '')
+        self.assertEqual(result, b'')
 
     def test_it_with_no_params(self):
         def view(request):
@@ -180,7 +181,7 @@ class TestJSONRPCIntegration(unittest.TestCase):
                         params='{')
         self.assertEqual(resp.status_int, 200)
         self.assertEqual(resp.content_type, 'application/json')
-        result = json.loads(resp.body)
+        result = resp.json
         self.assertEqual(result['jsonrpc'], '2.0')
         self.assertEqual(result['error']['code'], -32700)
 
@@ -196,6 +197,36 @@ class TestJSONRPCIntegration(unittest.TestCase):
         result = self._callFUT(app, 'dummy', [2, 3])
         self.assertEqual(result['error']['code'], -32603)
 
+    def test_it_with_rpc_error(self):
+        from pyramid_rpc.jsonrpc import JsonRpcError
+        def view(request):
+            raise JsonRpcError(code=500, message='dummy')
+        config = self.config
+        config.include('pyramid_rpc.jsonrpc')
+        config.add_jsonrpc_endpoint('rpc', '/api/jsonrpc')
+        config.add_jsonrpc_method(view, endpoint='rpc', method='dummy')
+        app = config.make_wsgi_app()
+        app = TestApp(app)
+        result = self._callFUT(app, 'dummy', [])
+        self.assertEqual(result['error']['code'], 500)
+        self.assertEqual(result['error']['message'], 'dummy')
+        self.assertFalse('data' in result['error'])
+
+    def test_it_with_rpc_error_with_data(self):
+        from pyramid_rpc.jsonrpc import JsonRpcError
+        def view(request):
+            raise JsonRpcError(code=500, message='dummy', data='foo')
+        config = self.config
+        config.include('pyramid_rpc.jsonrpc')
+        config.add_jsonrpc_endpoint('rpc', '/api/jsonrpc')
+        config.add_jsonrpc_method(view, endpoint='rpc', method='dummy')
+        app = config.make_wsgi_app()
+        app = TestApp(app)
+        result = self._callFUT(app, 'dummy', [])
+        self.assertEqual(result['error']['code'], 500)
+        self.assertEqual(result['error']['message'], 'dummy')
+        self.assertEqual(result['error']['data'], 'foo')
+
     def test_it_with_cls_view(self):
         class view(object):
             def __init__(self, request):
@@ -264,97 +295,15 @@ class TestJSONRPCIntegration(unittest.TestCase):
         result = self._callFUT(app, 'dummy', [2, 3, 4])
         self.assertEqual(result['error']['code'], -32602)
 
-class DummyAuthenticationPolicy(object):
-    userid = None
-    groups = ()
-
-    def authenticated_userid(self, request):
-        return self.userid
-
-    def effective_principals(self, request):
-        from pyramid.security import Everyone
-        from pyramid.security import Authenticated
-        principals = [Everyone]
-        userid = self.authenticated_userid(request)
-        if userid:
-            principals += [Authenticated]
-            principals += [userid]
-            principals += self.groups
-        return principals
-
-class TestJSONRPCFixture(unittest.TestCase):
-    package = 'pyramid_rpc.tests.fixtures.jsonrpc'
-
-    def _callFUT(self, app, method, params, id=5, version='2.0',
-                 path='/api/jsonrpc', content_type='application/json'):
-        body = {}
-        if id is not None:
-            body['id'] = id
-        if version is not None:
-            body['jsonrpc'] = version
-        if method is not None:
-            body['method'] = method
-        if params is not None:
-            body['params'] = params
-        resp = app.post(path, content_type=content_type,
-                        params=json.dumps(body))
-        self.assertEqual(resp.status_int, 200)
-        self.assertEqual(resp.content_type, 'application/json')
-        result = json.loads(resp.body)
-        self.assertEqual(result['jsonrpc'], '2.0')
-        self.assertEqual(result['id'], id)
-        return result
-
-    def setUp(self):
-        from pyramid.config import Configurator
-        from pyramid.authorization import ACLAuthorizationPolicy
-        config = Configurator(package=self.package)
-        config.include(self.package)
-        self.auth_policy = DummyAuthenticationPolicy()
-        config.set_authentication_policy(self.auth_policy)
-        config.set_authorization_policy(ACLAuthorizationPolicy())
+    def test_it_error_with_no_id(self):
+        def view(request):
+            raise Exception
+        config = self.config
+        config.include('pyramid_rpc.jsonrpc')
+        config.add_jsonrpc_endpoint('rpc', '/api/jsonrpc')
+        config.add_jsonrpc_method(view, endpoint='rpc', method='dummy')
         app = config.make_wsgi_app()
-        self.testapp = TestApp(app)
-        self.config = config
-
-    def tearDown(self):
-        testing.tearDown()
-
-    def test_basic(self):
-        result = self._callFUT(self.testapp, 'basic', [])
-        self.assertEqual(result['result'], 'basic')
-
-    def test_exc(self):
-        result = self._callFUT(self.testapp, 'exc', [])
+        app = TestApp(app)
+        result = self._callFUT(app, 'dummy', [], id=None, expect_error=True)
         self.assertEqual(result['error']['code'], -32603)
 
-    def test_decorated_method(self):
-        result = self._callFUT(self.testapp, 'create', [1, 2])
-        self.assertEqual(result['result'], {'create':'bob'})
-
-    def test_decorated_class_method(self):
-        result = self._callFUT(self.testapp, 'say_hello', ['bob'])
-        self.assertEqual(result['result'], 'hello, bob')
-
-    def test_secure_hello_with_credentials(self):
-        self.auth_policy.userid = 'bob'
-        result = self._callFUT(self.testapp, 'secure_hello', ['bob'],
-                               path='/api/jsonrpc/secure')
-        self.assertEqual(result['result'], 'hello, bob')
-
-    def test_secure_hello_with_no_credentials(self):
-        result = self._callFUT(self.testapp, 'secure_hello', ['bob'],
-                               path='/api/jsonrpc/secure')
-        self.assertEqual(result['result'], 'hello, bob')
-
-    def test_secure_echo_with_credentials(self):
-        self.auth_policy.userid = 'bob'
-        result = self._callFUT(self.testapp, 'secure_echo', ['foo'],
-                               path='/api/jsonrpc/secure')
-        self.assertEqual(result['result'], 'foo')
-
-    def test_secure_echo_with_no_credentials(self):
-        result = self._callFUT(self.testapp, 'secure_echo', ['bob'],
-                               path='/api/jsonrpc/secure')
-        self.assertEqual(result['error']['code'], -32600)
-
