@@ -1,17 +1,14 @@
 import logging
 
-import xmlrpclib
-
+import venusian
 from pyramid.exceptions import ConfigurationError
 from pyramid.httpexceptions import HTTPNotFound
 from pyramid.response import Response
-from pyramid.view import view_config
+from pyramid.security import NO_PERMISSION_REQUIRED
 
-from zope.deprecation import deprecated
-
-from pyramid_rpc.api import view_lookup
 from pyramid_rpc.api import MapplyViewMapper
 from pyramid_rpc.api import ViewMapperArgsInvalid
+from pyramid_rpc.compat import xmlrpclib
 
 
 log = logging.getLogger(__name__)
@@ -45,100 +42,6 @@ class XmlRpcParseError(XmlRpcError):
     faultString = 'parse error; not well formed'
 
 
-def xmlrpc_marshal(data):
-    """ Marshal a Python data structure into an XML document suitable
-    for use as an XML-RPC response and return the document.  If
-    ``data`` is an ``xmlrpclib.Fault`` instance, it will be marshalled
-    into a suitable XML-RPC fault response."""
-    if isinstance(data, xmlrpclib.Fault):
-        return xmlrpclib.dumps(data)
-    else:
-        return xmlrpclib.dumps((data,),  methodresponse=True)
-
-
-def parse_xmlrpc_request(request):
-    """ Deserialize the body of a request from an XML-RPC request
-    document into a set of params and return a two-tuple.  The first
-    element in the tuple is the method params as a sequence, the
-    second element in the tuple is the method name."""
-    if request.content_length > (1 << 23):
-        # protect from DOS (> 8MB body)
-        raise ValueError('Body too large (%s bytes)' % request.content_length)
-    params, method = xmlrpclib.loads(request.body)
-    return params, method
-
-
-class xmlrpc_view(object):
-    """ This decorator may be used with pyramid view callables to enable them
-    to repsond to XML-RPC method calls.
-
-    If ``method`` is not supplied, then the callable name will be used for
-    the method name. If ``route_name`` is not supplied, it is assumed that
-    the appropriate route was added to the application's config (named
-    'RPC2').
-
-    """
-    def __init__(self, method=None, route_name='RPC2'):
-        self.method = method
-        self.route_name = route_name
-
-    def __call__(self, wrapped, view_config=view_config):
-        # view_config passable for unit testing purposes only
-        method_name = self.method or wrapped.__name__
-        try:
-            # pyramid 1.1
-            from pyramid.renderers import null_renderer
-            renderer = null_renderer
-        except ImportError:  # pragma: no cover
-            # pyramid 1.0
-            renderer = None
-        return view_config(route_name=self.route_name, name=method_name,
-                           renderer=renderer)(wrapped)
-
-
-def xmlrpc_endpoint(request):
-    """A base view to be used with add_route to setup an XML-RPC dispatch
-    endpoint
-
-    Use this view with ``add_route`` to setup an XML-RPC endpoint, for
-    example::
-
-        config.add_route('RPC2', '/apis/RPC2', view=xmlrpc_endpoint)
-
-    XML-RPC methods should then be registered with ``add_view`` using the
-    route_name of the endpoint, the name as the xmlrpc method name. Or for
-    brevity, the :class:`~pyramid_rpc.xmlrpc.xmlrpc_view` decorator can be
-    used.
-
-    For example, to register an xmlrpc method 'list_users'::
-
-        @xmlrpc_view()
-        def list_users(request):
-            args = request.rpc_args
-            return {'users': [...]}
-
-    Existing views that return a dict can be used with xmlrpc_view.
-
-    """
-    params, method = parse_xmlrpc_request(request)
-    request.rpc_args = request.xmlrpc_args = params  # b/w compat xmlrpc_args
-
-    view_callable = view_lookup(request, method=method)
-    if not view_callable:
-        return HTTPNotFound("No method of that name was found.")
-    else:
-        data = view_callable(request.context, request)
-        xml = xmlrpc_marshal(data)
-        response = Response(xml)
-        response.content_type = 'text/xml'
-        response.content_length = len(xml)
-        return response
-
-deprecated('xmlrpc_endpoint',
-           ('Deprecated as of pyramid_rpc 0.3, use the new API as described '
-            'in the documentation.'))
-
-
 def exception_view(exc, request):
     if isinstance(exc, xmlrpclib.Fault):
         fault = exc
@@ -207,7 +110,8 @@ def add_xmlrpc_endpoint(self, name, *args, **kw):
     predicates = kw.setdefault('custom_predicates', [])
     predicates.append(xmlrpc_endpoint_predicate)
     self.add_route(name, *args, **kw)
-    self.add_view(exception_view, route_name=name, context=Exception)
+    self.add_view(exception_view, route_name=name, context=Exception,
+                  permission=NO_PERMISSION_REQUIRED)
 
 
 def add_xmlrpc_method(self, view, **kw):
@@ -264,27 +168,24 @@ class xmlrpc_method(object):
 
     """
     def __init__(self, method=None, **kw):
-        endpoint = kw.pop('endpoint', kw.pop('route_name', None))
-        if endpoint is None:
-            raise ConfigurationError(
-                'Cannot register a XML-RPC endpoint without specifying the '
-                'name of the endpoint.')
-
-        kw.setdefault('mapper', MapplyViewMapper)
-        kw.setdefault('renderer', 'pyramid_rpc:xmlrpc')
-        kw['route_name'] = endpoint
         self.method = method
         self.kw = kw
 
     def __call__(self, wrapped):
-        method = self.method or wrapped.__name__
         kw = self.kw.copy()
+        kw['method'] = self.method or wrapped.__name__
 
-        def xmlrpc_method_predicate(context, request):
-            return getattr(request, 'rpc_method', None) == method
-        predicates = kw.setdefault('custom_predicates', [])
-        predicates.append(xmlrpc_method_predicate)
-        return view_config(**kw)(wrapped)
+        def callback(context, name, ob):
+            config = context.config.with_package(info.module)
+            config.add_xmlrpc_method(view=ob, **kw)
+
+        info = venusian.attach(wrapped, callback, category='pyramid')
+        if info.scope == 'class':
+            # ensure that attr is set if decorating a class method
+            kw.setdefault('attr', wrapped.__name__)
+
+        kw['_info'] = info.codeinfo # fbo action_method
+        return wrapped
 
 
 def includeme(config):
@@ -306,4 +207,5 @@ def includeme(config):
     config.add_directive('add_xmlrpc_endpoint', add_xmlrpc_endpoint)
     config.add_directive('add_xmlrpc_method', add_xmlrpc_method)
     config.add_renderer('pyramid_rpc:xmlrpc', xmlrpc_renderer)
-    config.add_view(exception_view, context=xmlrpclib.Fault)
+    config.add_view(exception_view, context=xmlrpclib.Fault,
+                    permission=NO_PERMISSION_REQUIRED)
